@@ -61,7 +61,7 @@ from typing import (
 )
 from unicodedata import normalize
 
-from natsort.compat.fastnumbers import fast_float, fast_int
+from natsort.compat.fastnumbers import try_float, try_int
 from natsort.compat.locale import (
     StrOrBytes,
     get_decimal_point,
@@ -111,7 +111,7 @@ NumTransformer = Callable[[Any], NumTransform]
 
 # For the string component transform factory
 StrBytesNum = Union[str, bytes, float, int]
-StrTransformer = Callable[[str], StrBytesNum]
+StrTransformer = Callable[[Iterable[str]], Iterator[StrBytesNum]]
 
 # For the final data transform factory
 FinalTransform = AnyTuple
@@ -329,7 +329,7 @@ def natsort_key(
     --------
     parse_string_factory
     parse_bytes_factory
-    parse_number_factory
+    parse_number_or_none_factory
 
     """
 
@@ -337,26 +337,17 @@ def natsort_key(
     if key is not None:
         val = key(val)
 
-    # Assume the input are strings, which is the most common case
-    try:
-        return string_func(cast(str, val))
-    except (TypeError, AttributeError):
-
-        # If bytes type, use the bytes_func
-        if type(val) in (bytes,):
-            return bytes_func(cast(bytes, val))
-
-        # Otherwise, assume it is an iterable that must be parsed recursively.
-        # Do not apply the key recursively.
-        try:
-            return tuple(
-                natsort_key(x, None, string_func, bytes_func, num_func)
-                for x in cast(Iterable[Any], val)
-            )
-
-        # If that failed, it must be a number.
-        except TypeError:
-            return num_func(val)
+    if isinstance(val, (str, PurePath)):
+        return string_func(val)
+    elif isinstance(val, bytes):
+        return bytes_func(val)
+    elif isinstance(val, Iterable):
+        # Must be parsed recursively, but do not apply the key recursively.
+        return tuple(
+            natsort_key(x, None, string_func, bytes_func, num_func) for x in val
+        )
+    else:  # Anything else goes here
+        return num_func(val)
 
 
 def parse_bytes_factory(alg: NSType) -> BytesTransformer:
@@ -426,10 +417,23 @@ def parse_number_or_none_factory(
     nan_replace = float("+inf") if alg & ns.NANLAST else float("-inf")
 
     def func(
-        val: Any, _nan_replace: float = nan_replace, _sep: StrOrBytes = sep
+        val: Any,
+        _nan_replace: float = nan_replace,
+        _sep: StrOrBytes = sep,
+        reverse: bool = nan_replace == float("+inf"),
     ) -> BasicTuple:
         """Given a number, place it in a tuple with a leading null string."""
-        return _sep, (_nan_replace if val != val or val is None else val)
+        # Add a trailing string numbers equaling _nan_replace. This will make
+        # the ordering between None NaN, and the NaN replacement value...
+        # None comes first, then NaN, then the replacement value.
+        if val != val:
+            return _sep, _nan_replace, "3" if reverse else "1"
+        elif val is None:
+            return _sep, _nan_replace, "2"
+        elif val == _nan_replace:
+            return _sep, _nan_replace, "1" if reverse else "3"
+        else:
+            return _sep, val
 
     # Return the function, possibly wrapping in tuple if PATH is selected.
     if alg & ns.PATH and alg & ns.UNGROUPLETTERS and alg & ns.LOCALEALPHA:
@@ -514,7 +518,7 @@ def parse_string_factory(
         c = compose_input(b)  # Decompose unicode if using LOCALE
         d = splitter(c)  # Split string into components.
         e = filter(None, d)  # Remove empty strings.
-        f = map(component_transform, e)  # Apply transform on components.
+        f = component_transform(e)  # Apply transform on components.
         g = sep_inserter(f, sep)  # Insert '' between numbers.
         return final_transform(g, original)  # Apply the final transform.
 
@@ -697,14 +701,14 @@ def string_component_transform_factory(alg: NSType) -> StrTransformer:
         func_chain.append(get_strxfrm())
 
     # Return the correct chained functions.
-    kwargs: Dict[str, Union[float, Callable[[str], StrOrBytes]]]
-    kwargs = {"key": chain_functions(func_chain)} if func_chain else {}
+    kwargs: Dict[str, Union[float, Callable[[str], StrOrBytes], bool]]
+    kwargs = {"on_fail": chain_functions(func_chain)} if func_chain else {}
+    kwargs["map"] = True
     if alg & ns.FLOAT:
-        # noinspection PyTypeChecker
         kwargs["nan"] = nan_val
-        return cast(Callable[[str], StrOrBytes], partial(fast_float, **kwargs))
+        return cast(StrTransformer, partial(try_float, **kwargs))
     else:
-        return cast(Callable[[str], StrOrBytes], partial(fast_int, **kwargs))
+        return cast(StrTransformer, partial(try_int, **kwargs))
 
 
 def final_data_transform_factory(
@@ -864,9 +868,9 @@ def do_decoding(s: Any, encoding: str) -> Any:
         *s* if *s* was not *bytes*.
 
     """
-    try:
-        return cast(bytes, s).decode(encoding)
-    except (AttributeError, TypeError):
+    if isinstance(s, bytes):
+        return s.decode(encoding)
+    else:
         return s
 
 
